#include <iostream>
#include <vector>
#include <random>
#include <cassert>
#include "testlib.h"

using namespace std;
using pi = pair<int, int>;

// O(q * ((k-1)/2)^k)

struct edge {
    int x, y, color, id;
};
vector <edge> edges;
map <pair <int, int>, int> exists;
vector <pair <int, int>> queries;

void add_edge(int x, int y, int color, int id) {
    exists[make_pair(x, y)] = exists[make_pair(y, x)] = 1;
    edges.push_back({x, y, color, id});
}

void gen_ktree(int k, int q, vector <int>& vertices) {
    const int n = k;
    const int m = n*(n-1)/2;
    assert(n % 2 == 1);

    vector<vector<int>> eid(n, vector<int>(n));
    vector<vector<int>> col(n, vector<int>(n));

    int cnt = 0;
    for (int i = 0; i < n; ++i) {
        for (int j = 0; j < i; ++j) {
            col[i][j] = col[j][i] = (i+j)%n + 1;
            eid[i][j] = eid[j][i] = ++cnt;
            add_edge(vertices[j], vertices[i], col[i][j], eid[i][j]);
        }
    }

    vector<pi> que;

    while (queries.size() < q) {
        int a = 0, b = 0;
        while (a == b) {
            a = rnd.next(1, k);
            b = rnd.next(1, k);
        }

        vector<int> edg_a, edg_b;
        for (int i = 0; i < n; ++i)
            for (int j = 0; j < i; ++j) {
                if (col[i][j] == a) edg_a.push_back(eid[i][j]), col[i][j] = col[j][i] = b;
                else if (col[i][j] == b) edg_b.push_back(eid[i][j]), col[i][j] = col[j][i] = a;
            }

        assert(2*edg_a.size() == k-1);
        for (int i = 0; i < (k-1)/2; ++i) {
            const int p = edg_a[i], q = edg_b[i];
            queries.emplace_back(p, b);
            queries.emplace_back(q, a);
        }
    }

    queries.resize(q);
}

struct dsu {
    vector <int> p, sz;

    dsu(int n) {
        p.resize(n);
        sz.resize(n, 1);
        for (int i = 0; i < n; ++i) {
            p[i] = i;
        }
    }

    int parent(int v) {
        if (v == p[v]) return v;
        return parent(p[v]);
    }

    inline bool unite(int a, int b) {
        a = parent(a);
        b = parent(b);
        if (a == b) return false;
        if (sz[a] > sz[b]) swap(a, b);
        p[a] = b;
        sz[b] += sz[a];
        return true;
    }
};

int main(int argc, char* argv[]) {
    registerGen(argc, argv, 1);
    int n = opt<int>("n");
    int m = opt<int>("m");
    int q = opt<int>("q");
    int c = opt<int>("c");
    assert(c % 2 == 0);
    vector <int> vertices;
    for (int i = 0; i < c - 1; ++i) {
        vertices.push_back(rnd.next(1, n));
    }
    gen_ktree(c - 1, q, vertices);

    int cur = 0;
    dsu now(n + 1);
    for (auto i : edges) {
        cur += now.unite(i.x, i.y);
    }
    while (cur < n - 1) {
        int x = rnd.next(1, n);
        int y = rnd.next(1, n);
        if (now.unite(x, y)) {
            add_edge(x, y, c, -1);
            cur++;
        }
    }
    while (edges.size() < m) {
        int x = rnd.next(1, n);
        int y = rnd.next(1, n);
        if (x == y || exists.count(make_pair(x, y))) {
            continue;
        }
        add_edge(x, y, c, -1);
    }
    mt19937 rn(rnd.next(1, 1000000));
    shuffle(edges.begin(), edges.end(), rn);
    vector <int> pos(m + 1);
    println(n, m, c);
    for (int i = 0; i < m; ++i) {
        println(edges[i].x, edges[i].y, edges[i].color);
        if (edges[i].id != -1) pos[edges[i].id] = i + 1;
    }
    println(q);
    for (auto i : queries) {
        println(pos[i.first], i.second);
    }
}
