#include <iostream>
#include <vector>
#include <iomanip>
#include <map>
#include <queue>
#include <set>
#include <array>
#include <deque>
#include <algorithm>
#include "testlib.h"
#include <random>

using namespace std;

struct dsu {
    vector <int> p, sz;

    dsu(int n) {
        p.resize(n);
        sz.resize(n, 1);
        for (int i = 0; i < n; ++i) {
            p[i] = i;
        }
    }

    int parent(int v) {
        if (v == p[v]) return v;
        return parent(p[v]);
    }

    inline bool unite(int a, int b) {
        a = parent(a);
        b = parent(b);
        if (a == b) return false;
        if (sz[a] > sz[b]) swap(a, b);
        p[a] = b;
        sz[b] += sz[a];
        return true;
    }
};

signed main(int argc, char** argv) {
    registerGen(argc, argv, 1);
    int n = opt<int>("n");
    int m = opt<int>("m");
    int c = opt<int>("c");
    int bucket = opt<int>("bucket");
    println(n, m, c);
    dsu now(n);
    vector <array <int, 3>> edges;
    int base_color = rnd.next(1, c);
    int cur = 0;
    map <pair <int, int>, int> exist;
    while (cur < n - 1) {
        int x = rnd.next(0, n - 1);
        int y = rnd.next(0, n - 1);
        if (now.unite(x, y)) {
            edges.push_back({x + 1, y + 1, base_color});
            exist[make_pair(x + 1, y + 1)] = exist[make_pair(y + 1, x + 1)] = 1;
            cur++;
        }
    }
    while (cur < m) {
        int x = rnd.next(1, n);
        int y = rnd.next(1, n);
        if (x != y && !exist.count(make_pair(x, y))) {
            exist[make_pair(x, y)] = exist[make_pair(y, x)] = 1;
            edges.push_back({x, y, base_color});
            cur++;
        }
    }
    for (auto i : edges) {
        println(i);
    }
    int q = opt<int>("q");
    println(q);
    for (int i = 0; i < q / bucket; ++i) {
        int xd = 0;
        int num = 0;
        for (auto j : edges) {
            num++;
            if (j[2] != base_color) {
                println(num, base_color);
                xd++;
            }
        }
        vector<int> vertices;
        for (int j = 0; j < 3; ++j) {
            vertices.push_back(rnd.next(1, n));
        }
        vector<int> may_change;
        num = 0;
        for (auto j : edges) {
            num++;
            bool ok = false;
            for (int k : vertices) {
                if (k == j[0] || k == j[1]) ok = true;
            }
            if (ok) may_change.push_back(num);
        }
        while (xd < bucket) {
            xd++;
            num = may_change[rnd.next(0, (int)may_change.size() - 1)];
            int col = rnd.next(1, c);
            println(num, col);
            edges[num - 1][2] = col;
        }
    }
}
