#include <iostream>
#include <vector>
#include <iomanip>
#include <map>
#include <queue>
#include <set>
#include <array>
#include <deque>
#include <algorithm>
#include "testlib.h"
#include <random>

using namespace std;

struct dsu {
    vector <int> p, sz;

    dsu(int n) {
        p.resize(n);
        sz.resize(n, 1);
        for (int i = 0; i < n; ++i) {
            p[i] = i;
        }
    }

    int parent(int v) {
        if (v == p[v]) return v;
        return parent(p[v]);
    }

    inline bool unite(int a, int b) {
        a = parent(a);
        b = parent(b);
        if (a == b) return false;
        if (sz[a] > sz[b]) swap(a, b);
        p[a] = b;
        sz[b] += sz[a];
        return true;
    }
};

signed main(int argc, char** argv) {
    registerGen(argc, argv, 1);
    int n = opt<int>("n");
    int m = opt<int>("m");
    int c = opt<int>("c");
    println(n, m, c);
    dsu now(n);
    vector <array <int, 3>> edges;
    int base_color = c;
    int cur = 0;
    map <pair <int, int>, int> exist;
    vector <int> vertices;
    while (vertices.size() < c + 4) {
        int x = rnd.next(1, n);
        bool ok = false;
        for (int i : vertices) {
            if (i == x) ok = true;
        }
        if (ok) continue;
        vertices.push_back(x);
    }
    vector <int> numbers;
    for (int i = 0; i < c + 4; ++i) {
        for (int j = i + 1; j < c + 4; ++j) {
            exist[make_pair(vertices[i], vertices[j])] = exist[make_pair(vertices[j], vertices[i])] = 1;
            numbers.push_back((int)edges.size());
            edges.push_back({vertices[i], vertices[j], rnd.next(1, c - 1)});
            cur += now.unite(vertices[i], vertices[j]);
        }
    }
    vector <int> free;
    for (int i : numbers) free.push_back(i);
    while (!free.empty()) {
        for (int i = 0; i < c - 1 && !free.empty(); ++i) {
            int pos = rnd.next(0, (int)free.size() - 1);
            edges[free[pos]][2] = i + 1;
            free.erase(free.begin() + pos);
        }
    }
    while (cur < n - 1) {
        int x = rnd.next(0, n - 1);
        int y = rnd.next(0, n - 1);
        if (now.unite(x, y)) {
            edges.push_back({x + 1, y + 1, base_color});
            exist[make_pair(x + 1, y + 1)] = exist[make_pair(y + 1, x + 1)] = 1;
            cur++;
        }
    }
    while (edges.size() < m) {
        int x = rnd.next(1, n);
        int y = rnd.next(1, n);
        if (x != y && !exist.count(make_pair(x, y))) {
            exist[make_pair(x, y)] = exist[make_pair(y, x)] = 1;
            edges.push_back({x, y, base_color});
            cur++;
        }
    }
    for (auto i : edges) {
        println(i);
    }
    int q = opt<int>("q");
    println(q);
    for (int i = 0; i < q / 2; ++i) {
        int num = rnd.next(0, (int)numbers.size() - 1);
        println(numbers[num] + 1, rnd.next(1, c));
        println(numbers[num] + 1, edges[numbers[num]][2]);
    }
}
