#include <iostream>
#include <vector>
#include <random>
#include <cassert>
#include "testlib.h"

using namespace std;
using pi = pair<int, int>;

// O(q * ((k-1)/2)^k)

struct edge {
    int x, y, color;
};
vector <edge> edges;
map <pair <int, int>, int> exists;

void add_edge(int x, int y, int color) {
    exists[make_pair(x, y)] = exists[make_pair(y, x)] = 1;
    edges.push_back({x, y, color});
}

void gen(vector <int>& vertices) {
    add_edge(vertices[0], vertices[1], 1);
    add_edge(vertices[1], vertices[2], 2);
    add_edge(vertices[2], vertices[0], 3);
    add_edge(vertices[3], vertices[4], 4);
    add_edge(vertices[4], vertices[5], 5);
    add_edge(vertices[5], vertices[3], 3);
}

struct dsu {
    vector <int> p, sz;

    dsu(int n) {
        p.resize(n);
        sz.resize(n, 1);
        for (int i = 0; i < n; ++i) {
            p[i] = i;
        }
    }

    int parent(int v) {
        if (v == p[v]) return v;
        return parent(p[v]);
    }

    inline bool unite(int a, int b) {
        a = parent(a);
        b = parent(b);
        if (a == b) return false;
        if (sz[a] > sz[b]) swap(a, b);
        p[a] = b;
        sz[b] += sz[a];
        return true;
    }
};

int main(int argc, char* argv[]) {
    registerGen(argc, argv, 1);
    int n = opt<int>("n");
    int m = opt<int>("m");
    int q = opt<int>("q");
    int c = opt<int>("c");
    vector <int> vertices;
    for (int i = 0; i < 6; ++i) {
        vertices.push_back(rnd.next(1, n));
    }
    gen(vertices);

    int cur = 0;
    dsu now(n + 1);
    for (auto i : edges) {
        cur += now.unite(i.x, i.y);
    }
    while (cur < n - 1) {
        int x = rnd.next(1, n);
        int y = rnd.next(1, n);
        if (now.unite(x, y)) {
            add_edge(x, y, rnd.next(6, c));
            cur++;
        }
    }
    while (edges.size() < m) {
        int x = rnd.next(1, n);
        int y = rnd.next(1, n);
        if (x == y || exists.count(make_pair(x, y))) {
            continue;
        }
        add_edge(x, y, rnd.next(6, c));
    }
    mt19937 rn(rnd.next(1, 1000000));
    shuffle(edges.begin(), edges.end(), rn);
    vector <int> pos(m + 1);
    println(n, m, c);
    for (int i = 0; i < m; ++i) {
        println(edges[i].x, edges[i].y, edges[i].color);
    }
    println(q);
    for (int i = 0; i < q; ++i) {
        println(rnd.next(1, m), rnd.next(6, c));
    }
}
