#include <bits/stdc++.h>

using namespace std;
using u32 = uint32_t;
using pi = pair<int, int>;

constexpr int N = 100'000;
constexpr int M = 100'000;
constexpr int K = 8;

struct CacheFriendlyDSU {
    int m;
    vector<int> hunite, haccess;
    vector<pi> hst;
    vector<int> id;
    vector<int> par, rnk;

    CacheFriendlyDSU(int m) : m{m}, hunite{}, haccess{}, id(m, -1), par{}, rnk{} {
    }

    int Access(int v) {
        if (id[v] != -1) return id[v];

        const int w = par.size();
        haccess.push_back(v);
        id[v] = w;
        par.push_back(w);
        rnk.push_back(1);
        return w;
    }

    int Root(int v) {
        v = Access(v);
        while (v != par[v]) v = par[v];
        return v;
    }

    bool Unite(int a, int b) {
        a = Root(a), b = Root(b);
        if (a == b) {
            return false;
        } else {
            if (rnk[a] < rnk[b]) swap(a, b);
            hunite.push_back(b);
            par[b] = a;
            rnk[a] += rnk[b];
            return true;
        }
    }

    void PushState() {
        hst.emplace_back(haccess.size(), hunite.size());
    }

    void PopState() {
        auto [a_sz, u_sz] = hst.back(); hst.pop_back();
        while (u_sz != hunite.size()) {
            const int b = hunite.back();
            const int a = par[b];
            par[b] = b;
            rnk[a] -= rnk[b];
            hunite.pop_back();
        }

        while (a_sz != haccess.size()) {
            const int v = haccess.back();
            id[v] = -1;
            par.pop_back();
            rnk.pop_back();
            haccess.pop_back();
        }
    }
};

struct ColorWheel {
    struct Wheel {
        vector<int> q;
        const vector<int>* col;
        vector<int>* inq;
        int qf, qe, n, this_col;

        void Resize(int n_) {
            n = n_;
            q.resize(n);
            qf = 0, qe = 0;
        }

        int Size() const {
            return qf <= qe ? qe - qf : qe - qf + n;
        }

        void Push(int v) {
            q[qe++] = v;
            if (qe == n) qe = 0;
        }

        int Next() {
            while (true) {
                const int v = q[qf++];
                if (qf == n) qf = 0;
                
                if (this_col == (*col)[v]) {
                    Push(v);
                    return v;
                } else {
                    (*inq)[v] ^= 1 << this_col;
                }
            }
        }
    };

    int n;
    vector<int> col, inq;
    array<int, K> cnt_col;
    array<Wheel, K> ws;

    ColorWheel(int n) : n(n), col(n, -1), inq(n), cnt_col(), ws() {
        fill(cnt_col.begin(), cnt_col.end(), 0);
        for (int i = 0; i < K; ++i) {
            ws[i].Resize(n+1);
            ws[i].col = &col;
            ws[i].inq = &inq;
            ws[i].this_col = i;
        }
    }

    void SetColor(int i, int c) {
        if (col[i] != -1) cnt_col[col[i]]--;
        
        col[i] = c;
        cnt_col[c]++;
        if ((~inq[i] >> c) & 1) {
            inq[i] |= 1 << c;
            ws[c].Push(i);
        }
    }
};

bool IsConnectivity(int n, const vector<pi>& edges) {
    vector<vector<int>> go(n);
    for (const auto& [u, v] : edges) {
        go[u].push_back(v);
        go[v].push_back(u);
    }

    queue<int> q;
    vector<char> was(n);

    q.push(0);
    was[0] = 1;

    while (!q.empty()) {
        int v = q.front(); q.pop();
        for (int to : go[v]) {
            if (was[to]) continue;
            was[to] = 1;
            q.push(to);
        }
    }
    return *min_element(was.begin(), was.end()) != 0;
}

void answer(bool cond) {
    cout << (cond ? "Yes" : "No") << '\n';
}

bool BruteAnswer(int i, const vector<vector<pi>>& edg, CacheFriendlyDSU& dsu) {
    if (i == edg.size()) return true;

    for (const auto& [a, b] : edg[i]) {
        dsu.PushState();
        if (dsu.Unite(a, b) && BruteAnswer(i+1, edg, dsu)) {
            dsu.PopState();
            return true;
        }
        dsu.PopState();
    }

    return false;
}

int main() {
    ios::sync_with_stdio(0); cin.tie(0);

    int n, m, k; cin >> n >> m >> k;

    CacheFriendlyDSU dsu(n);
    ColorWheel cw(m);

    vector<pi> edges(m);
    for (int i = 0; i < m; ++i) {
        int col = 0;
        cin >> edges[i].first >> edges[i].second >> col;
        --edges[i].first, --edges[i].second;
        cw.SetColor(i, col - 1);
    }

    vector<int> vertex_cache(n, -1);

    int q; cin >> q;
    if (!IsConnectivity(n, edges)) {
        for (int i = q; i--; ) answer(false);
    } else while (q--) {
        int eid, col; cin >> eid >> col;
        cw.SetColor(eid - 1, col - 1);

        bool color_zero = false;
        for (int i = 0; i < k; ++i) {
            if (cw.cnt_col[i] == 0) color_zero = true;
        }
        if (color_zero) {
            answer(false);
            continue;
        }

        vector<vector<pi>> important_edges(k);

        bool any_empty = false;
        for (int col = 0; col < k; ++col) {
            dsu.PushState();
            int seen = 0, added = 0;
            while (seen < cw.cnt_col[col] && added < k) {
                int eid = cw.ws[col].Next();
                auto [u, v] = edges[eid];
                if (dsu.Unite(u, v)) {
                    ++added;
                    important_edges[col].emplace_back(u, v);
                }
                ++seen;
            }
            if (added == 0) {
                any_empty = true;
            }

            dsu.PopState();
        }

        if (any_empty) {
            answer(false);
            continue;
        }

        sort(important_edges.begin(), important_edges.end(), [](auto& lhs, auto& rhs) {
            return lhs.size() < rhs.size();
        });

        while (!important_edges.empty() && important_edges.back().size() >= important_edges.size()) {
            important_edges.pop_back();
        }

        vector<int> cached_vertex;
        for (auto& e : important_edges) {
            for (auto& [a, b] : e) {
                if (vertex_cache[a] == -1) vertex_cache[a] = cached_vertex.size(), cached_vertex.push_back(a);
                a = vertex_cache[a];

                if (vertex_cache[b] == -1) vertex_cache[b] = cached_vertex.size(), cached_vertex.push_back(b);
                b = vertex_cache[b];
            }
        }
        for (int x : cached_vertex) vertex_cache[x] = -1;

        bool ans = BruteAnswer(0, important_edges, dsu);
        answer(ans);
    }
}
