#include <iostream>
#include <vector>
#include <iomanip>
#include <map>
#include <queue>
#include <set>
#include <array>
#include <deque>
#include <algorithm>

using namespace std;

struct dsu {
    vector <int> p;
    vector <int> changed, sz;

    dsu(int n) {
        p.resize(n);
        sz.resize(n, 1);
        for (int i = 0; i < n; ++i) p[i] = i;
    }

    int parent(int v) {
        if (p[v] == v) return v;
        return parent(p[v]);
    }

    inline bool unite(int a, int b) {
        a = parent(a);
        b = parent(b);
        if (a == b) return false;
        if (sz[a] > sz[b]) swap(a, b);
        changed.push_back(a);
        changed.push_back(b);
        sz[b] += sz[a];
        p[a] = b;
        return true;
    }

    inline void clear() {
        while (!changed.empty()) {
            sz[changed.back()] = 1;
            p[changed.back()] = changed.back();
            changed.pop_back();
        }
    }
};

bool gen(int cur, int max_color, vector <set <pair <int, int>>>& all, dsu& now, vector <pair <int, int>>& edges) {
    if (cur == max_color) {
        bool ok = true;
        for (auto i : edges) {
            if (!now.unite(i.first, i.second)) {
                ok = false;
                break;
            }
        }
        now.clear();
        return ok;
    }
    for (auto i : all[cur]) {
        edges.push_back(i);
        if (gen(cur + 1, max_color, all, now, edges)) return true;
        edges.pop_back();
    }
    return false;
}

inline void solve() {
    int n, m, max_color;
    cin >> n >> m >> max_color;
    vector <set <pair <int, int>>> all(max_color);
    vector <pair <int, int>> edges(m);
    vector <int> current_color(m);
    dsu now(n);
    int count_edges = 0;
    for (int i = 0; i < m; ++i) {
        cin >> edges[i].first >> edges[i].second >> current_color[i]; --edges[i].first; --edges[i].second; --current_color[i];
        all[current_color[i]].insert(edges[i]);
        count_edges += now.unite(edges[i].first, edges[i].second);
    }
    bool exist_spanning = true;
    if (count_edges < n - 1) {
        exist_spanning = false;
    }
    now.clear();
    int q;
    cin >> q;
    while (q--) {
        int num, c;
        cin >> num >> c; --num; --c;
        if (!exist_spanning) {
            cout << "No\n";
            continue;
        }
        int prev = current_color[num];
        all[prev].erase(edges[num]);
        current_color[num] = c;
        all[c].insert(edges[num]);
        vector <pair <int, int>> current_edges;
        if (gen(0, max_color, all, now, current_edges)) {
            cout << "Yes\n";
        } else {
            cout << "No\n";
        }
    }
}


signed main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    int t = 1;
    //cin >> t;
    while (t--) {
        solve();
    }
}
