#include <bits/stdc++.h>
using namespace std;

using ll = long long;
using ld = long double;

#define all(a) begin(a), end(a)
#define len(a) int((a).size())

struct disjoin_set_union {
    vector<int> par;
    vector<int> sz;
    vector<pair<int, int>> updates;

    disjoin_set_union(int n) : par(n), sz(n, 1) {
        iota(all(par), 0);
    }

    int root(int v) {
        while (par[v] != v) {
            v = par[v];
        }
        return v;
    }

    bool unite(int v, int u) {
        v = root(v);
        u = root(u);
        if (v == u) {
            return false;
        }

        if (sz[v] < sz[u]) {
            swap(v, u);
        }
        par[u] = v;
        sz[v] += sz[u];
        updates.emplace_back(v, u);
        return true;
    }

    void roll_back() {
        auto [v, u] = updates.back();
        updates.pop_back();
        sz[v] -= sz[u];
        par[u] = u;
    }
};

int main() {
    cin.tie(nullptr)->sync_with_stdio(false);

    int n, m, k;
    cin >> n >> m >> k;

    struct edge {
        int v;
        int u;
        int c;
    };
    vector<edge> edges(m);
    for (auto &[v, u, c] : edges) {
        cin >> v >> u >> c;
        v--, u--, c--;
    }

    vector<set<pair<int, int>>> g(k);
    for (int i = 0; i < m; i++) {
        g[edges[i].c].insert(minmax(edges[i].v, edges[i].u));
    }

    bool connected = true;
    {
        disjoin_set_union dsu(n);
        int components = n;
        for (const auto& [v, u, c] : edges) {
            components -= dsu.unite(v, u);
        }
        connected = components == 1;
    }

    disjoin_set_union dsu(n);

    auto solve = [&]() {
        if (!connected) {
            cout << "No\n";
            return;
        }
        if (k == 1) {
            cout << "Yes\n";
            return;
        }

        const int lim = k * k;
        vector<vector<pair<int, int>>> cands(k);
        for (int i = 0; i < k; i++) {
            auto it = g[i].begin();
            while (it != g[i].end() && len(cands[i]) < lim) {
                cands[i].push_back(*it);
                it++;
            }
        }

        bool found = false;

        std::function<void(int)> dfs = [&](int pos) {
            if (pos == k) {
                found = true;
                return;
            }
            for (const auto &[v, u] : cands[pos]) {
                if (dsu.unite(v, u)) {
                    dfs(pos + 1);
                    dsu.roll_back();
                    if (found) {
                        return;
                    }
                }
            }
        };

        dfs(0);
        cout << (found ? "Yes" : "No") << '\n';
    };

    int q;
    cin >> q;
    while (q--) {
        int id, c;
        cin >> id >> c;
        id--, c--;

        g[edges[id].c].erase(minmax(edges[id].v, edges[id].u));
        edges[id].c = c;
        g[edges[id].c].insert(minmax(edges[id].v, edges[id].u));

        solve();
    }
}
