#include <iostream>
#include <vector>
#include <iomanip>
#include <map>
#include <queue>
#include <set>
#include <array>
#include <deque>
#include <algorithm>

using namespace std;

struct dsu {
    vector <int> p, sz;
    vector <array <int, 3>> changes;

    dsu(int n) {
        p.resize(n);
        sz.resize(n, 1);
        for (int i = 0; i < n; ++i) {
            p[i] = i;
        }
    }

    int parent(int v) {
        if (v == p[v]) return v;
        return parent(p[v]);
    }

    inline bool unite(int a, int b) {
        a = parent(a);
        b = parent(b);
        if (a == b) return false;
        changes.push_back({a, p[a], sz[a]});
        changes.push_back({b, p[b], sz[b]});
        if (sz[a] > sz[b]) swap(a, b);
        p[a] = b;
        sz[b] += sz[a];
        return true;
    }

    inline void rem(int len) {
        while (changes.size() > len) {
            p[changes.back()[0]] = changes.back()[1];
            sz[changes.back()[0]] = changes.back()[2];
            changes.pop_back();
        }
    }
};

bool rec(int now, dsu& d, vector <set <pair <int, int>>>& all, int c) {
    if (now == c) return true;
    int start_len = d.changes.size();
    for (auto i : all[now]) {
        if (d.unite(i.first, i.second)) {
            if (rec(now + 1, d, all, c)) return true;
            d.rem(start_len);
        }
    }
    return false;
}

signed main(int argc, char** argv) {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    int n, m, c;
    cin >> n >> m >> c;
    vector <pair <int, int>> edges(m);
    vector <set <pair <int, int>>> all(c);
    vector <int> current_col(m);
    dsu now(n);
    int cnt_edges = 0;
    for (int i = 0; i < m; ++i) {
        cin >> edges[i].first >> edges[i].second >> current_col[i]; --edges[i].first; --edges[i].second; --current_col[i];
        all[current_col[i]].insert(edges[i]);
        cnt_edges += now.unite(edges[i].first, edges[i].second);
    }
    bool exist_spanning = true;
    if (cnt_edges < n - 1) {
        exist_spanning = false;
    }
    now.rem(0);

    int q;
    cin >> q;
    while (q--) {
        int num, col;
        cin >> num >> col; --num; --col;

        if (!exist_spanning) {
            cout << "NO\n";
            continue;
        }

        all[current_col[num]].erase(edges[num]);
        current_col[num] = col;
        all[current_col[num]].insert(edges[num]);

        if (rec(0, now, all, c)) cout << "Yes\n";
        else cout << "No\n";

        now.rem(0);
    }
}
